from rest_framework import serializers

from ichec_django_core.models import Notification

from .core import SERIALIZER_BASE_FIELDS


class NotificationSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        model = Notification
        fields = SERIALIZER_BASE_FIELDS + ("user", "read_time", "message", "link")
        read_only_fiels = SERIALIZER_BASE_FIELDS



from ichec_django_core.models import Member, Organization, Address

from ichec_django_core.test import (
    setup_default_users_and_groups,
    setup_default_organizations,
    add_group_permissions,
    AuthAPITestCase,
)


class TestOrganizationView(AuthAPITestCase):

    template = {
        "name": "My Org",
        "acronym": "MO",
        "description": "An description of the org",
        "address": {"line1": "1234 Street", "region": "Region", "country": "IE"},
        "website": "www.org.org",
    }
    update_template = {"address": {"country": "FR"}}

    def setUp(self):
        self.url = "/api/organizations/"

        setup_default_users_and_groups()

        add_group_permissions(
            "admins",
            Organization,
            ["change_organization", "add_organization", "delete_organization"],
        )

        add_group_permissions(
            "members",
            Organization,
            ["view_organization"],
        )

        orgs = setup_default_organizations(Member.objects.get(username="other_user"))
        self.internal_org, self.public_org, self.inactive_org = orgs

    def test_list_access(self):

        scenarios = (
            ["", 1, 200, "Public can list public items"],
            ["regular_user", 1, 200, "Registered can list public items"],
            ["member", 2, 200, "Member can list all items"],
            ["other_user", 2, 200, "Item member can list item"],
        )
        for user, count, status, msg in scenarios:
            if user:
                items = self.assert_status(self.auth_list(user), status, msg)
            else:
                items = self.assert_status(self.public_list(), status, msg)

            self.assertEqual(items["count"], count, msg)

    def test_detail_access(self):

        public = self.public_org.id
        internal = self.internal_org.id
        scenarios = (
            ["", public, 200, "Public can view public item"],
            ["", internal, 404, "Public can't view internal item"],
            ["regular_user", public, 200, "Registered can view public item"],
            ["regular_user", internal, 404, "Registered can't view internal item"],
            ["member", public, 200, "Member can view public item"],
            ["member", internal, 200, "Item Member can view internal item"],
            ["other_user", internal, 200, "Item member can view item"],
        )

        for user, item, status, msg in scenarios:
            if user:
                self.assert_status(self.auth_detail(user, item), status, msg)
            else:
                self.assert_status(self.detail(item), status, msg)

    def test_create_permissions(self):

        scenarios = (
            ["", 401, "Public can't create, no auth"],
            ["regular_user", 403, "Registered can't create, no perm"],
            ["admin_user", 201, "Admin can create"],
        )

        for user, status, msg in scenarios:
            if user:
                self.assert_status(self.auth_create(user, self.template), status, msg)
            else:
                self.assert_status(self.create(self.template), status, msg)

    def test_update_permissions(self):

        scenarios = (
            ["", 401, False, "Public can't update, no auth"],
            ["regular_user", 404, False, "Registered can't update, no list"],
            ["other_user", 200, True, "Item member can update"],
            ["admin_user", 200, False, "Admin can update"],
        )

        for user, status, member, msg in scenarios:
            if member:
                created = self.create_for_update("admin_user", [user])
            else:
                created = self.create_for_update("admin_user")

            updated = None
            if user:
                updated = self.assert_status(
                    self.auth_update(user, created["id"], created), status, msg
                )
            else:
                self.assert_status(self.update(created["id"], created), status, msg)

            if updated:
                self.verify_update(updated)

    def test_delete_permissions(self):

        scenarios = (
            ["", 401, False, "Public can't delete, no auth"],
            ["regular_user", 403, False, "Registered can't delete"],
            ["other_user", 403, True, "Item member can't delete"],
            ["admin_user", 204, False, "Admin can delete"],
        )

        for user, status, member, msg in scenarios:
            if member:
                created = self.create_for_update("admin_user", [user])
            else:
                created = self.create_for_update("admin_user")

            if user:
                self.assert_status(self.auth_delete(user, created["id"]), status, msg)
            else:
                self.assert_status(self.delete(created["id"]), status, msg)

    def test_filters(self):
        pass

    def test_fields(self):
        pass



import unicodedata
import logging

from django.conf import settings
from django.contrib.auth.models import Group
from django.core.exceptions import SuspiciousOperation
from django.urls import reverse

from mozilla_django_oidc.auth import OIDCAuthenticationBackend
from mozilla_django_oidc.utils import absolutify, import_from_settings

from ..models.member import Member

logger = logging.getLogger(__name__)


def generate_username(email):

    return unicodedata.normalize("NFKC", email.split("@")[0][:150])


def store_tokens(session, access_token, id_token, refresh_token):
    if import_from_settings("OIDC_STORE_ACCESS_TOKEN", False):
        session["oidc_access_token"] = access_token

    if import_from_settings("OIDC_STORE_ID_TOKEN", False):
        session["oidc_id_token"] = id_token

    if import_from_settings("OIDC_STORE_REFRESH_TOKEN", True):
        session["oidc_refresh_token"] = refresh_token


class OIDCAuthenticationBackendWithRefresh(OIDCAuthenticationBackend):
    """
    Modified version based on https://github.com/mozilla/mozilla-django-oidc/pull/377
    to add refresh token storage as part of the authentication process.

    They can be used later in the session refresh middleware

    """

    def authenticate(self, request, **kwargs):
        """Authenticates a user based on the OIDC code flow."""

        self.request = request
        if not self.request:
            return None

        state = self.request.GET.get("state")
        code = self.request.GET.get("code")
        nonce = kwargs.pop("nonce", None)
        code_verifier = kwargs.pop("code_verifier", None)

        if not code or not state:
            return None

        reverse_url = self.get_settings(
            "OIDC_AUTHENTICATION_CALLBACK_URL", "oidc_authentication_callback"
        )

        token_payload = {
            "client_id": self.OIDC_RP_CLIENT_ID,
            "client_secret": self.OIDC_RP_CLIENT_SECRET,
            "grant_type": "authorization_code",
            "code": code,
            "redirect_uri": absolutify(self.request, reverse(reverse_url)),
        }

        if code_verifier is not None:
            token_payload.update({"code_verifier": code_verifier})

        # Get the token
        token_info = self.get_token(token_payload)
        id_token = token_info.get("id_token")
        access_token = token_info.get("access_token")
        refresh_token = token_info.get("refresh_token")

        # Validate the token
        payload = self.verify_token(id_token, nonce=nonce)
        if payload:
            session = self.request.session
            store_tokens(session, access_token, id_token, refresh_token)
            try:
                return self.get_or_create_user(access_token, id_token, payload)
            except SuspiciousOperation as exc:
                logger.warning("Failed to get or create user: %s", exc)
                return None

        return None


class MemberOIDCAuthenticationBackend(OIDCAuthenticationBackendWithRefresh):

    def __init__(self, *args, **kwargs):
        self.default_scopes = ("openid", "email", "profile")
        super().__init__(self, args, kwargs)

    def get_scopes(self):
        env_settings = self.get_settings("OIDC_RP_SCOPES", None)
        if env_settings:
            return env_settings.split()
        return self.default_scopes

    def verify_claims(self, claims):
        if "email" not in claims:
            return False
        return True

    def get_username(self, claims):
        if "preferred_username" in claims:
            return claims["preferred_username"]
        return generate_username(claims.get("email"))

    def create_user(self, claims):
        user = super().create_user(claims)
        user.first_name = claims.get("given_name", "")
        user.last_name = claims.get("family_name", "")

        # Create an associated Member
        member = Member(user_ptr_id=user.pk)
        member.__dict__.update(user.__dict__)

        if settings.AUTO_ASSIGNED_USER_GROUP:

            # If there's an auto-assign group assign them
            group = Group.objects.filter(name=settings.AUTO_ASSIGNED_USER_GROUP).first()
            if group:
                member.groups.add(group)
            member.save()

        return user



from rest_framework import permissions

from ichec_django_core.models import Notification
from ichec_django_core.serializers import NotificationSerializer

from .core import BaseModelViewSet
from .permissions import SelfEditOrDjangoModelPermissions


class NotificationViewSet(BaseModelViewSet):

    queryset = Notification.objects.all()
    serializer_class = NotificationSerializer
    permission_classes = [
        permissions.IsAuthenticated,
        SelfEditOrDjangoModelPermissions,
    ]

    def filter_self(self, queryset):
        return queryset.filter(user__id=self.request.user.id)

    def get_queryset(self):
        """
        If the user has view permissions for this model then return all objects,
        otherwise filter by their own objects only.
        """

        queryset = Notification.objects.all().order_by("id")

        unread = self.request.query_params.get("unread")
        if unread is not None:
            queryset = queryset.filter(read_time=None)

        return self.filter_self_or_permission(queryset, Notification)



from django.db.models import Q

from rest_framework.viewsets import ModelViewSet

from ichec_django_core.models import Organization

from ichec_django_core.serializers import (
    OrganizationDetailSerializer,
    OrganizationPublicDetailSerializer,
    OrganizationListSerializer,
)

from .core import SearchableModelViewSet
from .permissions import PublicViewMemberEditOrDjangoModelPermissions


class OrganizationViewSet(SearchableModelViewSet):

    model = Organization
    queryset = Organization.objects.filter(active=True)
    serializer_class = OrganizationListSerializer
    permission_classes = [
        PublicViewMemberEditOrDjangoModelPermissions,
    ]
    ordering_fields = SearchableModelViewSet.ordering_fields + ("name",)
    ordering: tuple[str, ...] = ("name",)
    search_fields = ["name", "address__line1", "address__country", "address__region"]
    filterset_fields = ("members__id",)

    serializers = {
        "retrieve": OrganizationDetailSerializer,
        "public_retrieve": OrganizationPublicDetailSerializer,
        "list": OrganizationListSerializer,
        "create": OrganizationDetailSerializer,
        "update": OrganizationDetailSerializer,
        "partial_update": OrganizationDetailSerializer,
    }

    def get_queryset(self):
        """
        Public users can only see public Orgs
        Registered users can see pubic Orgs or orgs they are members of
        Users with view permission can see all Orgs
        """

        queryset = ModelViewSet.get_queryset(self)

        if not self.request.user.is_authenticated:
            return queryset.filter(public=True)

        if self.has_model_view_permission():
            return queryset

        return queryset.filter(
            Q(public=True) | Q(members__id=self.request.user.id)
        ).distinct()



# Generated by Django 5.2.1 on 2026-02-02 17:52

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ichec_django_core", "0010_alter_memberpreferences_member"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="memberpreferences",
            name="member",
        ),
        migrations.AddField(
            model_name="member",
            name="preferences",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="member",
                to="ichec_django_core.memberpreferences",
            ),
        ),
    ]



from rest_framework import permissions
from rest_framework.authtoken.views import ObtainAuthToken
from rest_framework.authtoken.models import Token
from rest_framework.response import Response
from rest_framework.generics import RetrieveAPIView
from rest_framework.decorators import action
from rest_framework.views import APIView

from ichec_django_core.models import Member

from ichec_django_core.serializers import (
    MemberListSerializer,
    MemberDetailSerializer,
    MemberSelfDetailSerializer,
)

from .core import SearchableModelViewSet
from .files import ObjectFileUploadView, DEFAULT_IMAGE_FORMATS
from .permissions import SelfEditOrDjangoModelPermissions


class MemberViewSet(SearchableModelViewSet):

    model = Member
    queryset = Member.objects.filter(active=True)
    serializer_class = MemberListSerializer
    permission_classes = [
        permissions.IsAuthenticated,
        SelfEditOrDjangoModelPermissions,
    ]
    ordering_fields = SearchableModelViewSet.ordering_fields + ("username",)
    ordering: tuple[str, ...] = ("username",)
    search_fields = ["username", "first_name", "last_name", "email"]
    filterset_fields = ("organizations__id", "groups__id")

    serializers = {
        "retrieve": MemberDetailSerializer,
        "list": MemberListSerializer,
        "create": MemberDetailSerializer,
        "update": MemberDetailSerializer,
        "partial_update": MemberDetailSerializer,
    }

    @action(detail=True, url_path="profile", url_name="profile")
    def profile(self, request, pk=None):
        return self.download(request, "profile")

    @action(detail=True, url_path="profile/thumbnail", url_name="profile-thumbnail")
    def profile_thumbnail(self, request, pk=None):
        return self.download(request, "profile_thumbnail")

    def get_serializer_class(self):
        if (
            self.action in ("retrieve", "update", "partial_update")
            and self.request.user.id == self.get_object().id
        ):
            return MemberSelfDetailSerializer
        return self.serializers.get(self.action, self.serializer_class)

    def get_queryset(self):
        queryset = self.queryset
        board_id = self.request.query_params.get("access_boards__id")
        if board_id is not None:
            queryset = queryset.filter(access_boards__id=board_id)
        return queryset


class MemberSelfView(RetrieveAPIView):

    serializer_class = MemberSelfDetailSerializer
    permission_classes = [
        permissions.IsAuthenticated,
    ]

    def get_object(self):
        return Member.objects.get(id=self.request.user.id)


class MemberProfileUploadView(ObjectFileUploadView):
    model = Member
    queryset = Member.objects.all()
    file_field = "profile"
    extensions = DEFAULT_IMAGE_FORMATS
    permission_classes = [
        permissions.IsAuthenticated,
        SelfEditOrDjangoModelPermissions,
    ]


class GetAuthToken(APIView):

    permission_classes = [permissions.IsAuthenticated]

    def get(self, request, *args, **kwargs):
        token = Token.objects.get(user__id=request.user.id)
        return Response({"token": token.key})


class CustomAuthToken(ObtainAuthToken):

    authentication_classes: list = []

    """
    This overrides DRFs built in api auth token view
    so we return some more user details. This helps clients
    fetch the user post auth.
    """

    def post(self, request, *args, **kwargs):
        serializer = self.serializer_class(
            data=request.data, context={"request": request}
        )
        serializer.is_valid(raise_exception=True)
        user = serializer.validated_data["user"]
        token, created = Token.objects.get_or_create(user=user)

        return Response({"token": token.key, "user_id": user.pk})



"""
Django settings for marinerg_tap project.

Generated by 'django-admin startproject' using Django 5.1.

For more information on this file, see
https://docs.djangoproject.com/en/5.1/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/5.1/ref/settings/
"""

from pathlib import Path
import os

from corsheaders.defaults import default_headers

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/5.1/howto/deployment/checklist/

secret_key_env = os.environ.get("DJANGO_SECRET_KEY")
if not secret_key_env:
    raise RuntimeError("No DJANGO_SECRET_KEY key set in environment")
SECRET_KEY = secret_key_env.replace('"', "")

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = bool(os.environ.get("DJANGO_DEBUG", default=0))

ALLOWED_HOSTS = os.environ.get("DJANGO_ALLOWED_HOSTS", default="*").split(",")

ADMIN_URL = os.environ.get("DJANGO_ADMIN_URL", default="admin")

API_AUTH_URL = os.environ.get("DJANGO_API_AUTH_URL", default="api-auth")

# OIDC Connection
WITH_OIDC = bool(os.environ.get("WITH_OIDC", default=0))
OIDC_RP_CLIENT_ID = os.environ.get("OIDC_RP_CLIENT_ID")
OIDC_RP_CLIENT_SECRET = os.environ.get("OIDC_RP_CLIENT_SECRET")
OIDC_RP_SIGN_ALGO = "RS256"
OIDC_USE_PKCE = True
OIDC_OP_AUTHORIZATION_ENDPOINT = os.environ.get("OIDC_OP_AUTHORIZATION_ENDPOINT")
OIDC_OP_TOKEN_ENDPOINT = os.environ.get("OIDC_OP_TOKEN_ENDPOINT")
OIDC_OP_USER_ENDPOINT = os.environ.get("OIDC_OP_USER_ENDPOINT")
OIDC_OP_JWKS_ENDPOINT = os.environ.get("OIDC_OP_JWKS_ENDPOINT")
OIDC_USERNAME_ALGO = "ichec_django_core.auth.backends.generate_username"

# SECURITY WARNING: This is meant just for local development - it will allow ANY user to
# create an account on the portal. Use a suitable third-party
# user management tool/IDP (e.g. Keycloak) in production.
WITH_USER_LOGIN = bool(os.environ.get("DJANGO_WITH_USER_LOGIN", default=0))

# If a new user is created during login - e.g. from OIDC they will be
# automatically added to this group
AUTO_ASSIGNED_USER_GROUP = os.environ.get("DJANGO_AUTO_ASSIGNED_USER_GROUP")

# If true users created by fixtures or the admin portal are automatically marked
# as verified. In production you likely want this to be false.
AUTO_VERIFY_USERS = bool(os.environ.get("DJANGO_AUTO_VERIFY_USERS", default=0))

# Switch to enable soft deleting on supported models.
SOFT_DELETE = bool(os.environ.get("DJANGO_SOFT_DELETE", default=1))

# Application definition
INSTALLED_APPS = [
    "ichec_django_core.apps.IchecDjangoCoreConfig",
    "rest_framework",
    "corsheaders",
    "django_countries",
    "django_prometheus",
    "django_filters",
    "rest_framework.authtoken",
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
]

if WITH_OIDC:
    INSTALLED_APPS.append("mozilla_django_oidc")
    LOGIN_REDIRECT_URL = os.environ.get("LOGIN_REDIRECT_URL", default="/")
    LOGOUT_REDIRECT_URL = os.environ.get("OIDC_LOGOUT_REDIRECT_URL", default="/")

WITH_VERBOSE_LOGGING = os.environ.get("DJANGO_APP_VERBOSE_LOGGING", default=False)

WITH_METRICS = os.environ.get("DJANGO_WITH_METRICS", default=False)

MIDDLEWARE = []
if WITH_VERBOSE_LOGGING:
    MIDDLEWARE.append("ichec_django_core.middleware.RequestLoggerMiddleware")

if WITH_METRICS:
    MIDDLEWARE.append("django_prometheus.middleware.PrometheusBeforeMiddleware")

MIDDLEWARE.extend(
    [
        "django.middleware.security.SecurityMiddleware",
        "django.contrib.sessions.middleware.SessionMiddleware",
        "corsheaders.middleware.CorsMiddleware",
        "django.middleware.common.CommonMiddleware",
        "django.middleware.csrf.CsrfViewMiddleware",
        "django.contrib.auth.middleware.AuthenticationMiddleware",
        "django.contrib.messages.middleware.MessageMiddleware",
        "django.middleware.clickjacking.XFrameOptionsMiddleware",
    ]
)

if WITH_OIDC:
    MIDDLEWARE.append("ichec_django_core.middleware.oidc.RefreshOIDCAccessToken")

if WITH_METRICS:
    MIDDLEWARE.append("django_prometheus.middleware.PrometheusAfterMiddleware")

if WITH_VERBOSE_LOGGING:
    MIDDLEWARE.append("ichec_django_core.middleware.RequestLoggerMiddleware")

# Auth
AUTHENTICATION_BACKENDS = ["django.contrib.auth.backends.ModelBackend"]
if WITH_OIDC:
    AUTHENTICATION_BACKENDS = [
        "ichec_django_core.auth.backends.MemberOIDCAuthenticationBackend"
    ] + AUTHENTICATION_BACKENDS

ROOT_URLCONF = "app.urls"


def get_templates(base_dir: Path):
    return [
        {
            "BACKEND": "django.template.backends.django.DjangoTemplates",
            "DIRS": [os.path.join(base_dir, "templates")],
            "APP_DIRS": True,
            "OPTIONS": {
                "context_processors": [
                    "django.template.context_processors.debug",
                    "django.template.context_processors.request",
                    "django.contrib.auth.context_processors.auth",
                    "django.contrib.messages.context_processors.messages",
                ],
            },
        },
    ]


TEMPLATES = get_templates(BASE_DIR)

WSGI_APPLICATION = "app.wsgi.application"


# Database
# https://docs.djangoproject.com/en/5.1/ref/settings/#databases


def get_databases(base_dir: Path):
    return {
        "default": {
            "ENGINE": os.environ.get("DJANGO_SQL_ENGINE", "django.db.backends.sqlite3"),
            "NAME": os.environ.get("DJANGO_SQL_DATABASE", base_dir / "db.sqlite3"),
            "USER": os.environ.get("DJANGO_SQL_USER", "user"),
            "PASSWORD": os.environ.get("DJANGO_SQL_PASSWORD", "password"),
            "HOST": os.environ.get("DJANGO_SQL_HOST", "localhost"),
            "PORT": os.environ.get("DJANGO_SQL_PORT", "5432"),
        }
    }


DATABASES = get_databases(BASE_DIR)


# Password validation
# https://docs.djangoproject.com/en/5.1/ref/settings/#auth-password-validators

pv_prefix = "django.contrib.auth.password_validation"
AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": f"{pv_prefix}.UserAttributeSimilarityValidator",
    },
    {
        "NAME": f"{pv_prefix}.MinimumLengthValidator",
    },
    {
        "NAME": f"{pv_prefix}.CommonPasswordValidator",
    },
    {
        "NAME": f"{pv_prefix}.NumericPasswordValidator",
    },
]


# Internationalization
# https://docs.djangoproject.com/en/5.1/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_TZ = True

SECURE_PROXY_SSL_HEADER = ("HTTP_X_FORWARDED_PROTO", "https")

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/5.1/howto/static-files/

STATIC_URL = "static/"


def get_static_root(base_dir: Path):
    return Path(os.environ.get("DJANGO_STATIC_ROOT", default=base_dir / "staticfiles"))


STATIC_ROOT = get_static_root(BASE_DIR)


def get_media_root(base_dir: Path):
    return Path(os.environ.get("DJANGO_MEDIA_ROOT", default=base_dir / "mediafiles"))


MEDIA_ROOT = get_media_root(BASE_DIR)

DEFAULT_FILE_STORAGE = "django_downloadview.storage.SignedStorage"
DOWNLOADVIEW_BACKEND = "django_downloadview.nginx.XAccelRedirectMiddleware"

DOWNLOADVIEW_RULES = [
    {
        "source_url": "/media/nginx/",
        "destination_url": "/nginx-optimized-by-middleware/",
    },
]

# Default primary key field type
# https://docs.djangoproject.com/en/5.1/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

DRF_AUTH_CLASSES = []
if WITH_OIDC:
    DRF_AUTH_CLASSES.append("mozilla_django_oidc.contrib.drf.OIDCAuthentication")
DRF_AUTH_CLASSES.extend(
    [
        "rest_framework.authentication.TokenAuthentication",
        "rest_framework.authentication.SessionAuthentication",
    ]
)

REST_FRAMEWORK = {
    "DEFAULT_PERMISSION_CLASSES": [
        "rest_framework.permissions.IsAuthenticated",
    ],
    "DEFAULT_FILTER_BACKENDS": ("django_filters.rest_framework.DjangoFilterBackend",),
    "DEFAULT_AUTHENTICATION_CLASSES": DRF_AUTH_CLASSES,
    "DEFAULT_PAGINATION_CLASS": "ichec_django_core.views.pagination.PageNumberWithSizePagination",  # NOQA
    "PAGE_SIZE": 100,
}

if DEBUG:
    CORS_ALLOWED_ORIGINS = ["http://localhost:4200", "http://127.0.0.1:4200"]

    # Below is to allow Cross site setting of the credential cookies
    # Use it only in Debug mode for development.
    CORS_ALLOW_CREDENTIALS = True
    CSRF_COOKIE_SAMESITE = "Lax"
    SESSION_COOKIE_SAMESITE = "Lax"
    CSRF_TRUSTED_ORIGINS = ["http://localhost:4200"]
    CORS_ALLOW_HEADERS = [*default_headers, "Content-Disposition"]
    CORS_EXPOSE_HEADERS = ["Content-Disposition", "Content-Type", "X-CSRFToken"]
else:
    CSRF_COOKIE_SAMESITE = "Strict"
    CSRF_COOKIE_SECURE = True
    SESSION_COOKIE_SAMESITE = "Strict"
    SESSION_COOKIE_HTTPONLY = True
    SESSION_COOKIE_SECURE = True

if False:
    EMAIL_HOST_USER = "Test User"
    EMAIL_BACKEND = "django.core.mail.backends.filebased.EmailBackend"
    EMAIL_FILE_PATH = os.getcwd() + "/app-messages"
else:
    EMAIL_HOST = os.environ.get("DJANGO_EMAIL_HOST", default="")
    EMAIL_PORT = os.environ.get("DJANGO_EMAIL_PORT", default="")
    EMAIL_HOST_USER = os.environ.get("DJANGO_EMAIL_HOST_USER", default="")
    EMAIL_HOST_PASSWORD = os.environ.get("DJANGO_EMAIL_HOST_PASSWORD", default="")
    EMAIL_USE_TLS = True

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "handlers": {
        "console": {
            "class": "logging.StreamHandler",
        },
    },
    "root": {
        "handlers": ["console"],
        "level": "DEBUG",
    },
}



import calendar

from django.contrib.auth.models import Group
from django.http import JsonResponse
from django.conf import settings
from django.http import Http404, HttpResponseNotModified
from django.views.static import was_modified_since

from django_filters import rest_framework as df_filters
from rest_framework import permissions, viewsets, filters, status
from rest_framework.response import Response
from django_downloadview import DownloadResponse

from ichec_django_core.serializers import GroupSerializer

from .permissions import get_permission
from .files import upload_file

BASE_ORDERING_FIELDS = ("id", "created_at", "updated_at")


def server_manifest(_request):

    return JsonResponse(
        {
            "login_url": (
                "oidc/authenticate" if settings.WITH_OIDC else "accounts/login"
            ),
            "logout_url": "oidc/logout" if settings.WITH_OIDC else "api-auth/logout",
            "supports_cors": settings.DEBUG,
        }
    )


class BaseModelViewSet(viewsets.ModelViewSet):

    def filter_self_or_permission(self, queryset):
        if self.request.method == "GET":
            if self.request.user:
                if not self.has_model_view_permission():
                    return self.filter_self(queryset)
            else:
                return None
        return queryset

    def filter_self(self, queryset):
        return queryset

    def has_model_view_permission(self) -> bool:
        return self.self_has_permission(get_permission(self.model, "view"))

    def self_has_permission(self, perm: str) -> bool:
        return self.request.user.has_perm(perm)

    def get_queryset(self):
        """
        On classes that support public viewing filter by public instances
        on view sets.
        """

        queryset = super().get_queryset()
        if not self.request.user.is_authenticated:
            return queryset.filter(public=True)
        return queryset

    def get_serializer_class(self):
        if hasattr(self, "serializers"):
            if (
                not self.request.user.is_authenticated
                and "public_" + self.action in self.serializers
            ):
                return self.serializers["public_" + self.action]
            return self.serializers.get(self.action, self.serializer_class)
        return super().get_serializer_class()

    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)

        self.check_field_permissions(request, serializer)

        self.perform_create(serializer)
        headers = self.get_success_headers(serializer.data)
        return Response(
            serializer.data, status=status.HTTP_201_CREATED, headers=headers
        )

    def update(self, request, *args, **kwargs):

        partial = kwargs.pop("partial", False)
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=request.data, partial=partial)
        serializer.is_valid(raise_exception=True)

        self.check_field_permissions(request, serializer)

        self.perform_update(serializer)

        if getattr(instance, "_prefetched_objects_cache", None):
            # If 'prefetch_related' has been applied to a queryset, we need to
            # forcibly invalidate the prefetch cache on the instance.
            instance._prefetched_objects_cache = {}

        return Response(serializer.data)

    def perform_destroy(self, instance):
        """
        Override from DRF DestroyModelMixin. Intercept the delete request
        and if supported by the model type and settings do a soft delete.
        """
        if settings.SOFT_DELETE and hasattr(instance, "active"):
            setattr(instance, "active", False)
            instance.save()
        else:
            instance.delete()

    def check_field_permissions(self, _request, _serializer):
        pass

    def upload(self, request, field: str, extensions: tuple[str, ...]):
        """
        Allow upload to particular endpoints using DRF extra action decorators
        on ViewSets
        """

        return upload_file(
            request, self.get_object(), self.model._meta.model_name, field, extensions
        )

    def _was_modified_since(self, file_instance, since):
        try:
            return file_instance.was_modified_since(since)
        except (AttributeError, NotImplementedError):
            try:
                modification_time = calendar.timegm(
                    file_instance.modified_time.utctimetuple()
                )
            except (AttributeError, NotImplementedError) as e:
                print("!=======!", e)
                return True
            else:
                return was_modified_since(since, modification_time)

    def download(self, request, field):
        """
        Allow file download by DRF extra action decorators
        """

        instance = self.get_object()
        file_field = getattr(instance, field)
        if not file_field:
            raise Http404()

        since = request.headers.get("if-modified-since", None)
        if since is not None:
            if not self._was_modified_since(file_field, since):
                return HttpResponseNotModified()

        filename = file_field.name
        basename = str(field + "." + filename.split(".")[-1])
        return DownloadResponse(file_instance=file_field, basename=basename)


class SearchableModelViewSet(BaseModelViewSet):
    filter_backends = [
        df_filters.DjangoFilterBackend,
        filters.SearchFilter,
        filters.OrderingFilter,
    ]
    ordering_fields: tuple[str, ...] = BASE_ORDERING_FIELDS
    ordering: tuple[str, ...] = ("id",)


class GroupViewSet(SearchableModelViewSet):
    queryset = Group.objects.all()
    serializer_class = GroupSerializer
    permission_classes = [
        permissions.IsAuthenticated,
        permissions.DjangoModelPermissions,
    ]

    ordering_fields: tuple[str, ...] = ("name",)
    ordering: tuple[str, ...] = ("name",)
    search_fields: tuple[str, ...] = ("name",)

    def get_queryset(self):
        queryset = Group.objects.all()
        user_id = self.request.query_params.get("user")
        if user_id is not None:
            queryset = queryset.filter(user__id=user_id)
        return queryset



# Generated by Django 5.2.1 on 2026-02-01 12:30

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        (
            "ichec_django_core",
            "0008_organization_active_organization_deleted_at_and_more",
        ),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="organization",
            options={
                "verbose_name": "Organization",
                "verbose_name_plural": "Organizations",
            },
        ),
    ]



from django import forms
from django.contrib.auth.forms import UserCreationForm
from django.contrib.auth.models import User
from django.contrib.auth import login
from django.shortcuts import redirect, render


class RegistrationForm(UserCreationForm):
    email = forms.EmailField(required=True)

    class Meta:
        model = User
        fields = ["username", "email", "password"]


def register(request):
    if request.method == "POST":
        form = RegistrationForm(request.post)
        if form.is_valid():
            user = form.save()
            login(request, user)
            return redirect("/")
    else:
        form = RegistrationForm()
    return render(request, "registration/register.html", {"form": form})



try:
    from faker import Faker
    from faker_file.providers.image.pil_generator import PilImageGenerator
    from faker_file.providers.png_file import GraphicPngFileProvider

    _FAKER: Faker | None = Faker()
    if _FAKER:
        _FAKER.add_provider(GraphicPngFileProvider)
except ImportError:
    _FAKER = None

from .models import (  # NOQA
    ModelWithFiles,
    MemberCreate,
    MemberDetail,
    MemberIdentifierCreate,
    AddressCreate,
    OrganizationCreate,
    GroupCreate,
    FileWrapper,
)


def create_members(count: int = 100, offset: int = 0) -> list[ModelWithFiles]:

    content = []
    for idx in range(offset, count + offset):

        if _FAKER:
            profile = _FAKER.simple_profile()
            model = MemberCreate(
                username=str(profile["username"]),
                email=_FAKER.safe_email(),
                first_name=str(profile["name"]).split(" ")[0],
                last_name=str(profile["name"]).split(" ")[1],
                identifiers=[
                    MemberIdentifierCreate(id_type="ORCID", value="1234-5678-1234-5678")
                ],
            )
            image = _FAKER.graphic_png_file(
                image_generator_cls=PilImageGenerator, raw=True
            )
            profile_file = FileWrapper(filename=image.data["filename"], content=image)
            with open("test.png", "wb") as f:
                f.write(image)
            content.append(ModelWithFiles(model=model, files={"profile": profile_file}))
        else:
            model = MemberCreate(
                username=f"member_{idx}",
                email=f"member_{idx}@example.com",
                first_name="Script",
                last_name=f"User {idx}",
                identifiers=[
                    MemberIdentifierCreate(id_type="ORCID", value="1234-5678-1234-5678")
                ],
            )
            content.append(ModelWithFiles(model=model))
    return content


def create_groups(count: int = 100, offset: int = 0) -> list[GroupCreate]:
    content = []

    for idx in range(offset, count + offset):
        content.append(GroupCreate(name=f"Script Group {idx}"))
    return content


def create_organizations(
    count: int = 100, offset: int = 0, possible_members: tuple[MemberDetail, ...] = ()
) -> list[OrganizationCreate]:
    content = []

    for idx in range(offset, count + offset):

        if _FAKER:
            address = AddressCreate(
                line1=f"Building {_FAKER.building_number()}",
                line2=_FAKER.street_address(),
                city=_FAKER.city(),
                region="Region",
                postcode=str(_FAKER.postcode()),
                country=_FAKER.country_code(),
            )
        else:
            address = AddressCreate(
                line1="Apartment 123",
                line2="123 Street",
                city="City",
                region="Region",
                postcode="abc123",
                country="IE",
            )

        members = []
        if possible_members:
            num_members = len(possible_members) - 1
            ids = set(
                [
                    idx % num_members,
                    (idx + 1) % num_members,
                    (idx + 2) % num_members,
                    (idx + 4) % num_members,
                ]
            )
            members = [possible_members[jdx].url for jdx in ids]

        if _FAKER:
            item = OrganizationCreate(
                name=_FAKER.company(),
                acronym=f"ORG {idx}",
                description=_FAKER.paragraph(nb_sentences=5),
                address=address,
                website=_FAKER.safe_domain_name(),
                members=members,
            )
        else:
            item = OrganizationCreate(
                name=f"Organization {idx}",
                acronym=f"ORG {idx}",
                description=f"Description of org {idx}",
                address=address,
                website=f"www.org{idx}.com",
                members=members,
            )

        content.append(item)
    return content



# Generated by Django 5.2.1 on 2026-01-30 17:21

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "ichec_django_core",
            "0007_member_active_member_deleted_at_member_public_and_more",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="organization",
            name="active",
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name="organization",
            name="deleted_at",
            field=models.DateTimeField(default=django.utils.timezone.now, null=True),
        ),
        migrations.AddField(
            model_name="organization",
            name="public",
            field=models.BooleanField(default=False),
        ),
    ]



from rest_framework import serializers

from ..models import Form, FormField, FormFieldValue, FormGroup, PopulatedForm


class FormFieldSerializer(serializers.ModelSerializer):

    id = serializers.IntegerField(required=False)

    template = serializers.BooleanField(read_only=True)

    class Meta:
        model = FormField
        fields = (
            "id",
            "label",
            "key",
            "required",
            "description",
            "template",
            "options",
            "default",
            "field_type",
            "order",
        )
        read_only_fields = ("id",)

    def to_representation(self, instance):
        rep = super().to_representation(instance)
        rep["template"] = bool(instance.template)
        return rep


class FormGroupSerializer(serializers.ModelSerializer):

    fields = FormFieldSerializer(many=True)
    id = serializers.IntegerField(required=False)

    class Meta:
        model = FormGroup
        fields = (
            "id",
            "label",
            "description",
            "order",
            "fields",
        )
        read_only_fields = ("id",)

    def get_existing(self, id: str, collection: list):
        for c in collection:
            if c.id == id:
                return c
        raise RuntimeError("Requested non-existing entry")

    def update(self, instance, validated_data):

        fields = validated_data.pop("fields")

        instance = super().update(instance, validated_data)

        all_existing = instance.fields.all()

        # If any ids no longer exist delete them
        incoming_ids = [incoming["id"] for incoming in fields if "id" in incoming]
        existing_ids = [existing.id for existing in all_existing]
        for existing in existing_ids:
            if existing not in incoming_ids:
                FormField.objects.get(id=existing).delete()

        # If there is an id do an update, else do a create
        serializer = FormFieldSerializer()
        for field in fields:
            if "id" not in field:
                FormField.objects.create(group=instance, **field)
            else:
                serializer.update(self.get_existing(field["id"], all_existing), field)
        return instance


class FormSerializer(serializers.ModelSerializer):

    groups = FormGroupSerializer(many=True)
    id = serializers.IntegerField(required=False)

    class Meta:
        model = Form
        fields = ("groups", "id")
        read_only_fields = ("id",)

    def create(self, validated_data):
        groups = validated_data.pop("groups")

        instance = super().create(validated_data)

        for group in groups:
            fields = group.pop("fields")
            group_instance = FormGroup.objects.create(form=instance, **group)
            FormField.objects.bulk_create(
                [FormField(group=group_instance, **field) for field in fields]
            )
        return instance

    def get_existing(self, id: str, collection: list):
        for c in collection:
            if c.id == id:
                return c
        raise RuntimeError("Requested non-existing entry")

    def update(self, instance, validated_data):

        groups = validated_data.pop("groups")

        instance = super().update(instance, validated_data)

        all_existing = instance.groups.all()

        # If any ids no longer exist delete them
        incoming_ids = [incoming["id"] for incoming in groups if "id" in incoming]
        existing_ids = [existing.id for existing in all_existing]
        for existing in existing_ids:
            if existing not in incoming_ids:
                FormGroup.objects.get(id=existing).delete()

        # If there is an id do an update, else do a create
        serializer = FormGroupSerializer()
        for group in groups:
            if "id" not in group:
                fields = group.pop("fields")
                group_instance = FormGroup.objects.create(form=instance, **group)
                FormField.objects.bulk_create(
                    [FormField(group=group_instance, **field) for field in fields]
                )
            else:
                serializer.update(self.get_existing(group["id"], all_existing), group)
        return instance


class FormFieldValueDetailSerializer(serializers.ModelSerializer):

    field = FormFieldSerializer(read_only=True)
    id = serializers.IntegerField(required=False)
    asset = serializers.BooleanField(read_only=True)

    class Meta:
        model = FormFieldValue
        fields = ("id", "value", "field", "asset")
        read_only_fields = ("id", "field")

    def to_representation(self, instance):
        rep = super().to_representation(instance)
        rep["asset"] = bool(instance.asset)
        return rep


class FormFieldValueSerializer(serializers.ModelSerializer):

    id = serializers.IntegerField(required=False)

    class Meta:
        model = FormFieldValue
        fields = ("id", "value", "field")
        read_only_fields = ("id",)

    def to_representation(self, instance):
        return FormFieldValueDetailSerializer(context=self.context).to_representation(
            instance
        )


class PopulatedFormSerializer(serializers.ModelSerializer):

    values = FormFieldValueSerializer(many=True)
    id = serializers.IntegerField(required=False)

    class Meta:
        model = PopulatedForm
        fields = ("id", "values")
        read_only_fields = ("id",)

    def create(self, validated_data):
        values = validated_data.pop("values")

        instance = super().create(validated_data)

        FormFieldValue.objects.bulk_create(
            [FormFieldValue(form=instance, **value) for value in values]
        )
        return instance

    def get_existing(self, id: str, collection: list):
        for c in collection:
            if c.id == id:
                return c
        raise RuntimeError("Requested non-existing entry")

    def update(self, instance, validated_data):

        values = validated_data.pop("values")

        instance = super().update(instance, validated_data)

        all_existing = instance.values.all()

        # If any ids no longer exist delete them
        incoming_ids = [incoming["id"] for incoming in values if "id" in incoming]
        existing_ids = [existing.id for existing in all_existing]
        for existing in existing_ids:
            if existing not in incoming_ids:
                FormFieldValue.objects.get(id=existing).delete()

        # If there is an id do an update, else do a create
        serializer = FormFieldValueSerializer()
        for value in values:
            if "id" not in value:
                FormFieldValue.objects.create(form=instance, **value)
            else:
                serializer.update(self.get_existing(value["id"], all_existing), value)
        return instance



# Generated by Django 5.2.1 on 2026-01-30 17:14

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ichec_django_core", "0006_alter_memberpreferences_notifications"),
    ]

    operations = [
        migrations.AddField(
            model_name="member",
            name="active",
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name="member",
            name="deleted_at",
            field=models.DateTimeField(default=django.utils.timezone.now, null=True),
        ),
        migrations.AddField(
            model_name="member",
            name="public",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="member",
            name="verified",
            field=models.BooleanField(
                default=False,
                help_text="Used to indicate that a user is regarded as 'verified' - for example  by verifying their email address. This allows the concept of  users who have been invited to a portal - but can't yet interact  with it in most ways. A user coming from an OIDC provider might be assumed verified - based on properties of that provider.",
            ),
        ),
    ]



import typing

from django.db import models
from django.utils import timezone


class TimesStampMixin(models.Model):
    created_at = models.DateTimeField(default=timezone.now, editable=False)
    updated_at = models.DateTimeField(default=timezone.now)

    class Meta:
        abstract = True


class SoftDeleteMixin(models.Model):
    """
    A resource can be soft-deleted by setting active to false and setting a
    deletion time.

    This is filtered on in Querysets to disable resource visibility and interaction,
    but the resource still remains.

    Resources can be 'un-deleted' by resetting the active field.

    You could run a scheduled task to do full delete after some time period.

    Typing disabled as MyPy doesn't like duplicate Meta class definition
    but needed to stop Django generating DB fields
    """

    active = models.BooleanField(default=True)
    deleted_at = models.DateTimeField(default=timezone.now, null=True)

    class Meta:
        abstract = True


@typing.no_type_check
class DefaultPrivateMixin(models.Model):
    """
    Allows a resource to be set to being 'public'. In that case
    it can be viewed by anonymous users.

    Typing disabled as MyPy doesn't like duplicate Meta class definition
    but needed to stop Django generating DB fields
    """

    public = models.BooleanField(default=False)

    class Meta:
        abstract = True


@typing.no_type_check
class DefaultPublicMixin(models.Model):
    """
    Allows a resource to be set to being 'public'. In that case
    it can be viewed by anonymous users.

    Typing disabled as MyPy doesn't like duplicate Meta class definition
    but needed to stop Django generating DB fields
    """

    public = models.BooleanField(default=True)

    class Meta:
        abstract = True



# Generated by Django 5.2.1 on 2026-01-20 15:59

import django.db.models.deletion
import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        (
            "ichec_django_core",
            "0004_alter_member_profile_alter_member_profile_thumbnail",
        ),
    ]

    operations = [
        migrations.AddField(
            model_name="organization",
            name="email",
            field=models.CharField(max_length=200, null=True),
        ),
        migrations.CreateModel(
            name="MemberPreferences",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("notifications", models.JSONField(blank=True)),
                (
                    "member",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="ichec_django_core.member",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Notification",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now, editable=False
                    ),
                ),
                ("updated_at", models.DateTimeField(default=django.utils.timezone.now)),
                ("message", models.CharField(max_length=1000)),
                ("link", models.CharField(max_length=100)),
                ("read_time", models.DateTimeField(null=True)),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="ichec_django_core.member",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
        ),
    ]



from .member import Member, MemberIdentifier, MemberPreferences
from .feedback import Feedback
from .organization import Organization, Address
from .form import Form, FormGroup, FormField, FormFieldValue, PopulatedForm
from .utils import (
    make_zip,
    content_file_name,
)
from .mixins import (
    TimesStampMixin,
    SoftDeleteMixin,
    DefaultPrivateMixin,
    DefaultPublicMixin,
)
from .notification import Notification

__all__ = [
    "Organization",
    "Member",
    "MemberIdentifier",
    "MemberPreferences",
    "Address",
    "Feedback",
    "Notification",
    "Form",
    "FormGroup",
    "FormField",
    "FormFieldValue",
    "PopulatedForm",
    "SoftDeleteMixin",
    "DefaultPublicMixin",
    "DefaultPrivateMixin",
    "TimesStampMixin",
    "make_zip",
    "content_file_name",
]



from django.db import models

import functools

from .mixins import TimesStampMixin
from .utils import content_file_name


class FormFieldType(models.TextChoices):

    BOOLEAN = "BOOLEAN", "Boolean"
    CHAR = "CHAR", "Short Text"
    TEXT = "TEXT", "Long Text"
    RICH_TEXT = "RICH_TEXT", "Rich Text"
    SELECTION = "SELECTION", "Selection"
    INTEGER = "INTEGER", "Integer"
    FILE = "FILE", "File"


class Form(TimesStampMixin):

    class Meta:
        verbose_name = "Form"
        verbose_name_plural = "Forms"


class FormGroup(models.Model):

    class Meta:
        verbose_name = "Form Group"
        verbose_name_plural = "Form Groups"

    label = models.CharField(max_length=400, null=True)
    description = models.TextField(null=True)
    order = models.IntegerField(default=0)
    form = models.ForeignKey(Form, on_delete=models.CASCADE, related_name="groups")

    def __str__(self):
        return self.label if self.label else f"Form Group {self.id}"


class FormField(models.Model):

    class Meta:
        verbose_name = "Form Field"
        verbose_name_plural = "Form Fields"

    label = models.CharField(max_length=400)
    key = models.CharField(max_length=200)
    required = models.BooleanField(default=False)
    order = models.IntegerField(default=0)

    field_type = models.CharField(
        max_length=10,
        choices=FormFieldType.choices,
        default=FormFieldType.CHAR,
    )

    description = models.TextField(blank=True)

    template = models.FileField(
        null=True,
        upload_to=functools.partial(content_file_name, "form_field_template"),
    )

    default = models.CharField(max_length=500, blank=True)
    options = models.CharField(max_length=1000, null=True)

    group = models.ForeignKey(
        FormGroup, on_delete=models.CASCADE, related_name="fields"
    )

    def __str__(self):
        return self.label


class PopulatedForm(TimesStampMixin):

    class Meta:
        verbose_name = "Populated Form"
        verbose_name_plural = "Populated Forms"


class FormFieldValue(models.Model):

    class Meta:
        verbose_name = "Form Field Value"
        verbose_name_plural = "Form Field Values"

    value = models.CharField(max_length=500, blank=True)
    asset = models.FileField(
        null=True,
        upload_to=functools.partial(content_file_name, "form_field_value"),
    )
    field = models.ForeignKey(
        FormField, on_delete=models.CASCADE, related_name="values"
    )
    form = models.ForeignKey(
        PopulatedForm, on_delete=models.CASCADE, related_name="values"
    )

    def __str__(self):
        return self.value



# Generated by Django 5.2.1 on 2026-01-20 16:10

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ichec_django_core", "0005_organization_email_memberpreferences_notification"),
    ]

    operations = [
        migrations.AlterField(
            model_name="memberpreferences",
            name="notifications",
            field=models.JSONField(null=True),
        ),
    ]



from django.db import models

from .member import Member
from .mixins import TimesStampMixin


class Notification(TimesStampMixin):
    message = models.CharField(max_length=1000)
    link = models.CharField(max_length=100)
    read_time = models.DateTimeField(null=True)
    user = models.ForeignKey(Member, on_delete=models.CASCADE)



from rest_framework import permissions

from ichec_django_core.models import Feedback, Member

from ichec_django_core.serializers import FeedbackSerializer

from .permissions import OwnerFullOrDjangoModelPermissions
from .core import BaseModelViewSet


class CreatorFullOrDjangoModelPermissions(OwnerFullOrDjangoModelPermissions):
    owner_field = "creator"


class FeedbackViewSet(BaseModelViewSet):

    model = Feedback
    queryset = Feedback.objects.all().order_by("id")
    serializer_class = FeedbackSerializer
    permission_classes = [
        permissions.IsAuthenticated,
        CreatorFullOrDjangoModelPermissions,
    ]
    filterset_fields = ("creator__id",)

    def filter_self(self, queryset):
        return queryset.filter(creator__id=self.request.user.id)

    def get_queryset(self):
        """
        If the user has view permissions for this model then return all objects,
        otherwise filter by their own objects only.
        """
        return self.filter_self_or_permission(super().get_queryset())

    def perform_create(self, serializer):
        serializer.save(creator=Member.objects.get(id=self.request.user.id))



from django.urls import include, path
from django.contrib import admin
from django.contrib.auth import views as auth_views
from django.conf import settings

from .views import (
    MemberViewSet,
    MemberSelfView,
    MemberProfileUploadView,
    GroupViewSet,
    OrganizationViewSet,
    FeedbackViewSet,
    NotificationViewSet,
    CustomAuthToken,
    GetAuthToken,
)
from .views.core import server_manifest
from .views.login import register


def register_drf_views(router):
    router.register(r"groups", GroupViewSet)
    router.register(r"organizations", OrganizationViewSet)
    router.register(r"members", MemberViewSet)
    router.register(r"feedback", FeedbackViewSet)
    router.register(r"notifications", NotificationViewSet)
    return router


urlpatterns = [
    path("api-token-auth/", CustomAuthToken.as_view()),
    path(f"{settings.API_AUTH_URL}/", include("rest_framework.urls")),
    path(f"{settings.ADMIN_URL}/", admin.site.urls),
    path(
        r"manifest",
        server_manifest,
        name="manifest",
    ),
    path(
        r"api/self/",
        MemberSelfView.as_view(),
        name="member_self",
    ),
    path(
        r"api/token/",
        GetAuthToken.as_view(),
        name="token",
    ),
    path(
        r"api/members/<int:pk>/profile/upload",
        MemberProfileUploadView.as_view(),
        name="member_profiles_upload",
    ),
    path("", include("django_prometheus.urls")),
]

if settings.WITH_OIDC:
    urlpatterns += [path("oidc/", include("mozilla_django_oidc.urls"))]

if settings.WITH_USER_LOGIN:
    urlpatterns += [
        path(
            "accounts/login",
            auth_views.LoginView.as_view(
                template_name="ichec_django_core/registration/login.html"
            ),
        ),
        path("accounts/", include("django.contrib.auth.urls")),
        path("accounts/register", register, name="register"),
    ]



# Generated by Django 5.2.10 on 2026-02-02 08:43

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("ichec_django_core", "0009_alter_organization_options"),
    ]

    operations = [
        migrations.AlterField(
            model_name="memberpreferences",
            name="member",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE,
                related_name="preferences",
                to="ichec_django_core.member",
            ),
        ),
    ]



import PIL

from django.contrib.auth.models import Group, Permission
from django.contrib.contenttypes.models import ContentType

from ichec_django_core.models import Member, Organization, Address


def generate_image():
    return PIL.Image.linear_gradient("L")


def _add_member(name: str, groups: list[str] | None = None):
    member = Member.objects.create(username=name)

    if not groups:
        return member

    for group_name in groups:
        group = Group.objects.get(name=group_name)
        member.groups.add(group)
        member.save()
    return member


def setup_default_users_and_groups():

    for group in ("admins", "members", "regular_users"):
        Group.objects.create(name=group)

    regular_user = _add_member("regular_user")
    other_user = _add_member("other_user")

    member = _add_member("member", ["members"])
    other_member = _add_member("other_member", ["members"])

    admin = _add_member("admin_user", ["admins", "members"])

    return regular_user, other_user, member, other_member, admin


def add_group_permissions(group_name: str, model: type, permissions: list):

    group = Group.objects.get(name=group_name)
    content_type = ContentType.objects.get_for_model(model)
    for permission_name in permissions:
        permission = Permission.objects.get(
            codename=permission_name, content_type=content_type
        )
        group.permissions.add(permission)
    group.save()


def setup_default_address():
    return Address.objects.create(
        line1="123 Street",
        line2="123 Avenue",
        line3="123 District",
        city="Big City",
        region="Region",
        postcode="ABC123",
        country="IE",
    )


def setup_default_organizations(member):

    internal = Organization.objects.create(
        name="Test Org 1",
        acronym="TO1",
        email="email@testorg.com",
        description="description of org",
        address=setup_default_address(),
        website="www.org.org",
        public=False,
    )
    internal.members.add(member)

    public = Organization.objects.create(
        name="Test Org 2",
        acronym="TO2",
        email="email@testorg.com",
        description="description of org",
        address=setup_default_address(),
        website="www.org.org",
        public=True,
    )

    inactive = Organization.objects.create(
        name="Test Org 3",
        acronym="TO3",
        email="email@testorg.com",
        description="description of org",
        address=setup_default_address(),
        website="www.org.org",
        public=True,
        active=False,
    )

    return internal, public, inactive



from functools import partial

from django.contrib.auth.models import User
from rest_framework import status
from rest_framework.test import APITestCase

from ichec_django_core.models import Member


class AuthAPITestCase(APITestCase):
    def authenticate(self, username: str):
        user = User.objects.get(username=username)
        self.client.force_authenticate(user=user)

    def deauthenticate(self):
        self.client.force_authenticate(user=None)

    def authenticated_op(self, username: str, op):
        self.authenticate(username)
        response = op()
        self.deauthenticate()
        return response

    def get_member_url(self, name):
        user = User.objects.get(username=name)
        self.authenticate(name)
        resp = self.client.get(f"/api/members/{user.id}/", format="json")
        self.deauthenticate()
        return resp.data["url"]

    def get_user(self, name: str) -> Member:
        return Member.objects.get(username=name)

    def public_list(self):
        return self.client.get(self.url, format="json")

    def auth_list(self, username: str):
        return self.authenticated_op(username, self.public_list)

    def detail(self, resource_id: int):
        return self.client.get(self.url + f"{resource_id}/", format="json")

    def auth_detail(self, username: str, resource_id: int):
        return self.authenticated_op(username, partial(self.detail, resource_id))

    def create(self, data: dict, format="json"):
        return self.client.post(self.url, data, format=format)

    def auth_create(self, username: str, data: dict, format="json"):
        return self.authenticated_op(username, partial(self.create, data, format))

    def update(self, resource_id: int, data: dict, format="json"):
        return self.client.put(self.url + f"{resource_id}/", data, format=format)

    def auth_update(self, username: str, resource_id: int, data: dict):
        return self.authenticated_op(username, partial(self.update, resource_id, data))

    def delete(self, resource_id: int):
        return self.client.delete(self.url + f"{resource_id}/")

    def auth_delete(self, username: str, resource_id: int):
        return self.authenticated_op(username, partial(self.delete, resource_id))

    def put_file(self, resource_id: int, field: str, data, filename):
        return self.client.put(
            f"{self.url}{resource_id}/{field}/upload",
            data,
            headers={"Content-Disposition": f'attachment; filename="{filename}"'},
        )

    def auth_put_file(
        self, username: str, resource_id: int, field: str, data, filename: str
    ):
        return self.authenticated_op(
            username, partial(self.put_file, resource_id, field, data, filename)
        )

    def assert_status(self, response, code, msg=""):
        match code:
            case 200:
                return self.assert_200(response, msg)
            case 201:
                return self.assert_201(response, msg)
            case 204:
                self.assert_204(response, msg)
            case 401:
                self.assert_401(response, msg)
            case 403:
                self.assert_403(response, msg)
            case 404:
                self.assert_404(response, msg)

    def assert_200(self, response, msg=""):
        self.assertEqual(
            response.status_code,
            status.HTTP_200_OK,
            msg + " | Response: " + str(response.data),
        )
        return response.data

    def assert_201(self, response, msg=""):
        self.assertEqual(
            response.status_code,
            status.HTTP_201_CREATED,
            msg + " | Response: " + str(response.data),
        )
        return response.data

    def assert_204(self, response, msg=""):
        self.assertEqual(
            response.status_code,
            status.HTTP_204_NO_CONTENT,
            msg + " | Response: " + str(response.data),
        )

    def assert_401(self, response, msg=""):
        self.assertEqual(
            response.status_code,
            status.HTTP_401_UNAUTHORIZED,
            msg + " | Response: " + str(response.data),
        )

    def assert_403(self, response, msg=""):
        self.assertEqual(
            response.status_code,
            status.HTTP_403_FORBIDDEN,
            msg + " | Response: " + str(response.data),
        )

    def assert_404(self, response, msg=""):
        self.assertEqual(
            response.status_code,
            status.HTTP_404_NOT_FOUND,
            msg + " | Response: " + str(response.data),
        )

    def create_for_update(
        self, creator, members: list | None = None, member_key: str = "members"
    ):

        template = self.template
        if members:
            template[member_key] = [self.get_member_url(m) for m in members]

        created = self.assert_201(self.auth_create(creator, self.template))
        for key, value in self.update_template.items():
            if isinstance(value, dict):
                for sub_key, sub_value in value.items():
                    created[key][sub_key] = sub_value
        return created

    def verify_update(self, updated):
        for key, value in self.update_template.items():
            if isinstance(value, dict):
                for sub_key, sub_value in value.items():
                    self.assertEqual(updated[key][sub_key], sub_value)



import logging
import json
import time
from urllib.parse import quote, urlencode

import requests
from django.utils.crypto import get_random_string
from django.urls import reverse
from django.http import HttpResponseRedirect, JsonResponse

from mozilla_django_oidc.middleware import SessionRefresh
from mozilla_django_oidc.utils import (
    absolutify,
    add_state_and_verifier_and_nonce_to_session,
    import_from_settings,
    generate_code_challenge,
)


from ..auth.backends import store_tokens

logger = logging.getLogger(__name__)


class RefreshOIDCAccessToken(SessionRefresh):
    """
    The Mozilla OIDC SessionRefresh is a fairly hacky way to
    handle expiry of the OP user session. This version
    is adapted from this PR: https://github.com/mozilla/mozilla-django-oidc/pull/377
    to use a refresh token instead.

    Refer to: https://auth0.com/docs/tokens/refresh-token/current for flow.
    """

    def is_expired(self, request):
        if not self.is_refreshable_url(request):
            logger.debug("Request is not refreshable")
            return False

        expiration = request.session.get("oidc_id_token_expiration", 0)
        now = time.time()
        if expiration > now:
            # The id_token is still valid, so we don't have to do anything.
            logger.debug("Id token is still valid (%s > %s)", expiration, now)
            return False

        return True

    def _prepare_reauthorization(self, request):
        # Constructs a new authorization grant request to refresh the session.
        # Besides constructing the request, the state and nonce included in the
        # request are registered in the current session in preparation for the
        # client following through with the authorization flow.
        auth_url = self.OIDC_OP_AUTHORIZATION_ENDPOINT
        client_id = self.OIDC_RP_CLIENT_ID
        state = get_random_string(self.OIDC_STATE_SIZE)

        # Build the parameters as if we were doing a real auth handoff, except
        # we also include prompt=none.
        params = {
            "response_type": "code",
            "client_id": client_id,
            "redirect_uri": absolutify(
                request, reverse(self.OIDC_AUTHENTICATION_CALLBACK_URL)
            ),
            "state": state,
            "scope": self.OIDC_RP_SCOPES,
            "prompt": "none",
        }

        params.update(self.get_settings("OIDC_AUTH_REQUEST_EXTRA_PARAMS", {}))

        if self.get_settings("OIDC_USE_PKCE", False):
            code_verifier_length = self.get_settings("OIDC_PKCE_CODE_VERIFIER_SIZE", 64)
            # Check that code_verifier_length is between the min and max length
            # defined in https://datatracker.ietf.org/doc/html/rfc7636#section-4.1
            if not (43 <= code_verifier_length <= 128):
                raise ValueError("code_verifier_length must be between 43 and 128")

            # Generate code_verifier and code_challenge pair
            code_verifier = get_random_string(code_verifier_length)
            code_challenge_method = self.get_settings(
                "OIDC_PKCE_CODE_CHALLENGE_METHOD", "S256"
            )
            code_challenge = generate_code_challenge(
                code_verifier, code_challenge_method
            )

            # Append code_challenge to authentication request parameters
            params.update(
                {
                    "code_challenge": code_challenge,
                    "code_challenge_method": code_challenge_method,
                }
            )
        else:
            code_verifier = None

        if self.OIDC_USE_NONCE:
            nonce = get_random_string(self.OIDC_NONCE_SIZE)
            params.update({"nonce": nonce})

        # Register the one-time parameters in the session
        add_state_and_verifier_and_nonce_to_session(
            request, state, params, code_verifier
        )
        request.session["oidc_login_next"] = request.get_full_path()

        query = urlencode(params, quote_via=quote)
        return "{auth_url}?{query}".format(auth_url=auth_url, query=query)

    def finish(self, request, prompt_reauth=True):
        """Finish request handling and handle sending downstream responses for XHR.

        This function should only be run if the session is determind to
        be expired.

        Almost all XHR request handling in client-side code struggles
        with redirects since redirecting to a page where the user
        is supposed to do something is extremely unlikely to work
        in an XHR request. Make a special response for these kinds
        of requests.

        The use of 403 Forbidden is to match the fact that this
        middleware doesn't really want the user in if they don't
        refresh their session.
        """
        default_response = None
        xhr_response_json = {"error": "the authentication session has expired"}
        if prompt_reauth:
            # The id_token has expired, so we have to re-authenticate silently.
            refresh_url = self._prepare_reauthorization(request)
            default_response = HttpResponseRedirect(refresh_url)
            xhr_response_json["refresh_url"] = refresh_url

        if request.headers.get("x-requested-with") == "XMLHttpRequest":
            xhr_response = JsonResponse(xhr_response_json, status=403)
            if "refresh_url" in xhr_response_json:
                xhr_response["refresh_url"] = xhr_response_json["refresh_url"]
            return xhr_response
        else:
            return default_response

    def process_request(self, request):
        if not self.is_expired(request):
            return

        token_url = import_from_settings("OIDC_OP_TOKEN_ENDPOINT")
        client_id = import_from_settings("OIDC_RP_CLIENT_ID")
        client_secret = import_from_settings("OIDC_RP_CLIENT_SECRET")
        refresh_token = request.session.get("oidc_refresh_token")

        if not refresh_token:
            logger.debug("no refresh token stored")
            return self.finish(request, prompt_reauth=True)

        token_payload = {
            "grant_type": "refresh_token",
            "client_id": client_id,
            "client_secret": client_secret,
            "refresh_token": refresh_token,
        }

        try:
            response = requests.post(
                token_url,
                auth=None,
                data=token_payload,
                verify=import_from_settings("OIDC_VERIFY_SSL", True),
            )
            response.raise_for_status()
            token_info = response.json()
        except requests.exceptions.Timeout:
            logger.debug("timed out refreshing access token")
            # Don't prompt for reauth as this could be a temporary problem
            return self.finish(request, prompt_reauth=False)
        except requests.exceptions.HTTPError as exc:
            status_code = exc.response.status_code
            logger.debug("http error %s when refreshing access token", status_code)
            # OAuth error response will be a 400 for various situations, including
            # an expired token.
            # https://datatracker.ietf.org/doc/html/rfc6749#section-5.2
            return self.finish(request, prompt_reauth=(status_code == 400))
        except json.JSONDecodeError:
            logger.debug("malformed response when refreshing access token")
            # Don't prompt for reauth as this could be a temporary problem
            return self.finish(request, prompt_reauth=False)
        except Exception as exc:
            logger.debug("unknown error occurred when refreshing access token: %s", exc)
            # Don't prompt for reauth as this could be a temporary problem
            return self.finish(request, prompt_reauth=False)

        # Until we can properly validate an ID token on the refresh response
        # per the spec[1], we intentionally drop the id_token.
        # https://openid.net/specs/openid-connect-core-1_0.html#RefreshTokenResponse
        id_token = None
        access_token = token_info.get("access_token")
        refresh_token = token_info.get("refresh_token")
        store_tokens(request.session, access_token, id_token, refresh_token)
